package com.jiuqi.util;

import java.math.BigInteger;
import java.util.Random;

public final class Guid implements Cloneable {
   private static final char[] HEX_CHARS = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
   private static final char[] BASE64_CHARS = new char[]{'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '='};
   public static int GUID_SIZE = 32;
   public static String emptyGuid = "00000000000000000000000000000000";
   private static Random rndGuid = new Random(Math.round(Math.random() * 10000.0));
   private byte[] data = new byte[16];

   public static String newGuid() {
      Guid T = new Guid();
      T.generate();
      return T.toString();
   }

   public void generate() {
      BigInteger BI1 = BigInteger.valueOf(System.currentTimeMillis());
      BigInteger BI2;
      synchronized(rndGuid) {
         BI2 = new BigInteger(64, rndGuid);
      }

      byte[] BA = BI1.toByteArray();

      int i;
      for(i = 0; i < 8 - BA.length; ++i) {
         this.data[i] = 0;
      }

      for(i = 0; i < BA.length; ++i) {
         this.data[8 - BA.length + i] = BA[i];
      }

      BA = BI2.toByteArray();

      for(i = 0; i < 8 - BA.length; ++i) {
         this.data[8 + i] = 0;
      }

      for(i = 0; i < BA.length; ++i) {
         if (BA.length - i <= 8) {
            this.data[16 - BA.length + i] = BA[i];
         }
      }

   }

   public Object clone() {
      Guid copy = new Guid();
      copy.data = (byte[])this.data.clone();
      return copy;
   }

   public boolean equals(Object o) {
      return o instanceof Guid ? this.data.equals(((Guid)o).data) : false;
   }

   public String toString() {
      StringBuffer str = new StringBuffer(32);

      for(int i = 0; i < this.data.length; ++i) {
         int code = this.data[i] >> 4 & 15;
         str.append(HEX_CHARS[code]);
         code = this.data[i] & 15;
         str.append(HEX_CHARS[code]);
      }

      return str.toString();
   }

   public String tobase64() {
      StringBuffer s = new StringBuffer("0000000000000000000000");
      int p = 0;
      int i = 0;

      int b;
      int t;
      byte b;
      while(i < 15) {
         b = this.data[i++];
         b = b >>> 2 & 63;
         s.setCharAt(p++, BASE64_CHARS[b]);
         b = this.data[i++];
         t = b;
         b = b >>> 4 & 15;
         t &= 3;
         t <<= 4;
         b |= t;
         s.setCharAt(p++, BASE64_CHARS[b]);
         b = this.data[i++];
         b = b >>> 6 & 3;
         t = b & 15;
         t <<= 2;
         b |= t;
         s.setCharAt(p++, BASE64_CHARS[b]);
         t &= 63;
         b = t;
         s.setCharAt(p++, BASE64_CHARS[b]);
      }

      b = this.data[i++];
      b = b >>> 2 & 63;
      s.setCharAt(p++, BASE64_CHARS[b]);
      b = 0;
      b = b >>> 4 & 15;
      t = b & 3;
      t <<= 4;
      b |= t;
      s.setCharAt(p++, BASE64_CHARS[b]);
      return new String(s);
   }
}
